@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * pkg-array.h - primitives for pkg array handling
  *
- * Copyright © 2009 Guillem Jover <guillem@debian.org>
+ * Copyright © 2009-2015 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef LIBDPKG_PKG_ARRAY_H
@@ -40,7 +40,15 @@ struct pkg_array {
 	struct pkginfo **pkgs;
 };
 
-void pkg_array_init_from_db(struct pkg_array *a);
+typedef struct pkginfo *pkg_mapper_func(const char *name);
+typedef void pkg_array_visitor_func(struct pkg_array *a, struct pkginfo *pkg,
+                                    void *pkg_data);
+
+void pkg_array_init_from_hash(struct pkg_array *a);
+void pkg_array_init_from_names(struct pkg_array *a, pkg_mapper_func *pkg_mapper,
+                               const char **pkg_names);
+void pkg_array_foreach(struct pkg_array *a, pkg_array_visitor_func *pkg_visitor,
+                       void *pkg_data);
 void pkg_array_sort(struct pkg_array *a, pkg_sorter_func *pkg_sort);
 void pkg_array_destroy(struct pkg_array *a);
 
