@@ -1,4 +1,5 @@
 # Copyright © 2007-2010 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2007-2009,2012-2015,2017-2018 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,17 +24,6 @@ our $VERSION = '0.01';
 
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
-use Dpkg::Path qw(find_command);
-use Dpkg::Arch qw(debarch_to_gnutriplet get_build_arch get_host_arch);
-use Dpkg::IPC;
-
-# Decide which objdump to call
-our $OBJDUMP = 'objdump';
-if (get_build_arch() ne get_host_arch()) {
-    my $od = debarch_to_gnutriplet(get_host_arch()) . '-objdump';
-    $OBJDUMP = $od if find_command($od);
-}
-
 
 sub new {
     my $this = shift;
@@ -110,6 +100,8 @@ use constant {
     ELF_MACH_OR1K           => 92,
     ELF_MACH_XTENSA         => 94,
     ELF_MACH_MICROBLAZE     => 189,
+    ELF_MACH_ARCV2          => 195,
+    ELF_MACH_LOONGARCH      => 258,
     ELF_MACH_AVR_OLD        => 0x1057,
     ELF_MACH_OR1K_OLD       => 0x8472,
     ELF_MACH_ALPHA          => 0x9026,
@@ -134,6 +126,11 @@ use constant {
 
     ELF_FLAG_IA64_ABI64     => 0x00000010,
 
+    ELF_FLAG_LOONGARCH_SOFT_FLOAT     => 0x00000001,
+    ELF_FLAG_LOONGARCH_SINGLE_FLOAT   => 0x00000002,
+    ELF_FLAG_LOONGARCH_DOUBLE_FLOAT   => 0x00000003,
+    ELF_FLAG_LOONGARCH_ABI_MASK       => 0x00000007,
+
     ELF_FLAG_MIPS_ABI2      => 0x00000020,
     ELF_FLAG_MIPS_32BIT     => 0x00000100,
     ELF_FLAG_MIPS_FP64      => 0x00000200,
@@ -167,6 +164,7 @@ my %elf_mach_map = (
 # behavior, and we do not drop dependencies.
 my %elf_flags_mask = (
     ELF_MACH_IA64()     => ELF_FLAG_IA64_ABI64,
+    ELF_MACH_LOONGARCH() => ELF_FLAG_LOONGARCH_ABI_MASK,
     ELF_MACH_MIPS()     => ELF_FLAG_MIPS_ABI_MASK | ELF_FLAG_MIPS_ABI2,
     ELF_MACH_PPC64()    => ELF_FLAG_PPC64_ABI64,
 );
@@ -244,9 +242,12 @@ package Dpkg::Shlibs::Objdump::Object;
 
 use strict;
 use warnings;
+use feature qw(state);
 
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
+use Dpkg::Path qw(find_command);
+use Dpkg::Arch qw(debarch_to_gnutriplet get_build_arch get_host_arch);
 
 sub new {
     my $this = shift;
@@ -268,9 +269,9 @@ sub reset {
 
     $self->{file} = '';
     $self->{id} = '';
-    $self->{SONAME} = '';
     $self->{HASH} = '';
     $self->{GNU_HASH} = '';
+    $self->{INTERP} = 0;
     $self->{SONAME} = '';
     $self->{NEEDED} = [];
     $self->{RPATH} = [];
@@ -281,6 +282,14 @@ sub reset {
     return $self;
 }
 
+sub _select_objdump {
+    # Decide which objdump to call
+    if (get_build_arch() ne get_host_arch()) {
+        my $od = debarch_to_gnutriplet(get_host_arch()) . '-objdump';
+        return $od if find_command($od);
+    }
+    return 'objdump';
+}
 
 sub analyze {
     my ($self, $file) = @_;
@@ -298,6 +307,7 @@ sub analyze {
         return;
     }
 
+    state $OBJDUMP = _select_objdump();
     local $ENV{LC_ALL} = 'C';
     open(my $objdump, '-|', $OBJDUMP, '-w', '-f', '-p', '-T', '-R', $file)
         or syserr(g_('cannot fork for %s'), $OBJDUMP);
@@ -325,7 +335,7 @@ sub parse_objdump_output {
 	    $section = 'dyninfo';
 	    next;
 	} elsif (/^Program Header:/) {
-	    $section = 'header';
+	    $section = 'program';
 	    next;
 	} elsif (/^Version definitions:/) {
 	    $section = 'verdef';
@@ -364,6 +374,10 @@ sub parse_objdump_output {
                     $self->{RPATH} = [ split /:/, $rpath ];
                 }
 	    }
+        } elsif ($section eq 'program') {
+            if (/^\s*INTERP\s+/) {
+                $self->{INTERP} = 1;
+            }
 	} elsif ($section eq 'none') {
 	    if (/^\s*.+:\s*file\s+format\s+(\S+)$/) {
 		$self->{format} = $1;
@@ -481,9 +495,24 @@ sub apply_relocations {
 	# We want to mark as undefined symbols those which are currently
 	# defined but that depend on a copy relocation
 	next if not $sym->{defined};
-	next if not exists $self->{dynrelocs}{$sym->{name}};
-	if ($self->{dynrelocs}{$sym->{name}} =~ /^R_.*_COPY$/) {
+
+        my @relocs;
+
+        # When objdump qualifies the symbol with a version it will use @ when
+        # the symbol is in an undefined section (which we discarded above, or
+        # @@ otherwise.
+        push @relocs, $sym->{name} . '@@' . $sym->{version} if $sym->{version};
+
+        # Symbols that are not versioned, or versioned but shown with objdump
+        # fopm binutils < 2.26, do not have a version appended.
+        push @relocs, $sym->{name};
+
+        foreach my $reloc (@relocs) {
+            next if not exists $self->{dynrelocs}{$reloc};
+            next if not $self->{dynrelocs}{$reloc} =~ /^R_.*_COPY$/;
+
 	    $sym->{defined} = 0;
+            last;
 	}
     }
 }
@@ -536,7 +565,8 @@ sub get_needed_libraries {
 
 sub is_executable {
     my $self = shift;
-    return exists $self->{flags}{EXEC_P} && $self->{flags}{EXEC_P};
+    return (exists $self->{flags}{EXEC_P} && $self->{flags}{EXEC_P}) ||
+           (exists $self->{INTERP} && $self->{INTERP});
 }
 
 sub is_public_library {
