@@ -143,7 +143,7 @@ void checkpath(void) {
   }
 
   if (warned)
-    forcibleerr(fc_badpath,
+    forcibleerr(FORCE_BAD_PATH,
                 P_("%d expected program not found in PATH or not executable\n%s",
                    "%d expected programs not found in PATH or not executable\n%s",
                    warned),
@@ -182,7 +182,7 @@ ignore_depends_possi(struct deppossi *possi)
 bool
 force_depends(struct deppossi *possi)
 {
-  return fc_depends ||
+  return in_force(FORCE_DEPENDS) ||
          ignore_depends_possi(possi) ||
          ignore_depends(possi->up->up);
 }
@@ -190,7 +190,7 @@ force_depends(struct deppossi *possi)
 bool
 force_breaks(struct deppossi *possi)
 {
-  return fc_breaks ||
+  return in_force(FORCE_BREAKS) ||
          ignore_depends_possi(possi) ||
          ignore_depends(possi->up->up);
 }
@@ -198,7 +198,7 @@ force_breaks(struct deppossi *possi)
 bool
 force_conflicts(struct deppossi *possi)
 {
-  return fc_conflicts;
+  return in_force(FORCE_CONFLICTS);
 }
 
 void clear_istobes(void) {
@@ -228,7 +228,7 @@ dir_has_conffiles(struct fsys_namenode *file, struct pkginfo *pkg)
         pkg_name(pkg, pnaw_always));
   namelen = strlen(file->name);
   for (conff= pkg->installed.conffiles; conff; conff= conff->next) {
-      if (conff->obsolete)
+      if (conff->obsolete || conff->remove_on_upgrade)
         continue;
       if (strncmp(file->name, conff->name, namelen) == 0 &&
           strlen(conff->name) > namelen && conff->name[namelen] == '/') {
@@ -349,8 +349,8 @@ void
 log_action(const char *action, struct pkginfo *pkg, struct pkgbin *pkgbin)
 {
   log_message("%s %s %s %s", action, pkgbin_name(pkg, pkgbin, pnaw_always),
-	      versiondescribe(&pkg->installed.version, vdew_nonambig),
-	      versiondescribe(&pkg->available.version, vdew_nonambig));
+              versiondescribe_c(&pkg->installed.version, vdew_nonambig),
+              versiondescribe_c(&pkg->available.version, vdew_nonambig));
   statusfd_send("processing: %s: %s", action,
                 pkgbin_name(pkg, pkgbin, pnaw_nonambig));
 }
